@@ -6,12 +6,12 @@
 /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    RADAU: Implicit runge-Kutta of order 5
    due to Hairer and Wanner, with stepsize control and dense output
-   
-   The C-wrappers that provide the interface between FORTRAN codes and R-code 
-   are: C_deriv_func_rad: interface with R-code "func", passes derivatives  
+
+   The C-wrappers that provide the interface between FORTRAN codes and R-code
+   are: C_deriv_func_rad: interface with R-code "func", passes derivatives
         C_deriv_out_rad : interface with R-code "func", passes derivatives +
                                                                output variables
-  
+
    C_deriv_func_forc_rad provides the interface between the function specified in
    a DLL and the integrator, in case there are forcing functions.
 
@@ -40,13 +40,13 @@
 void F77_NAME(radau5)( int *,
          void (*)(int *, double *, double *, double *, double *, int *), // func
 		     double *, double *, double *, double *,
-		     double *, double *, int *,  
- 	       void (*)(int *, double *, double *, int *, int *, 
+		     double *, double *, int *,
+ 	       void (*)(int *, double *, double *, int *, int *,
                     double *, int *, double *, int *),                   // jac
 		     int *, int *, int *,
  	       void (*)(int *, double *, int *, double *, int *),              // mas
 		     int *, int *, int *,
-         void (*)(int *, double *, double *, double *, double *,  
+         void (*)(int *, double *, double *, double *, double *,
 			            int *, int *, double *, int *, int *, double *),   // soloutrad
 		     int *, double *, int *, int *, int*, double *, int*, int*);
 
@@ -77,13 +77,13 @@ static void C_deriv_func_rad (int *neq, double *t, double *y,
 
   for (i = 0; i < *neq; i++)  REAL(Y)[i] = y[i];
 
-  PROTECT(Time = ScalarReal(*t));                  incr_N_Protect();
-  PROTECT(R_fcall = lang3(R_deriv_func,Time,Y));   incr_N_Protect();
-  PROTECT(ans = eval(R_fcall, R_envir));           incr_N_Protect();
+  PROTECT(Time = ScalarReal(*t));
+  PROTECT(R_fcall = lang3(R_deriv_func,Time,Y));
+  PROTECT(ans = eval(R_fcall, R_envir));
 
   for (i = 0; i < *neq; i++)   ydot[i] = REAL(ans)[i];
 
-  my_unprotect(3);
+  UNPROTECT(3);
 }
 
 /* mass matrix function                                                       */
@@ -94,44 +94,44 @@ static void C_mas_func_rad (int *neq, double *am, int *lmas,
   int i;
   SEXP NEQ, LM, R_fcall, ans;
 
-  PROTECT(NEQ = NEW_INTEGER(1));                  incr_N_Protect();
-  PROTECT(LM = NEW_INTEGER(1));                   incr_N_Protect();
+  PROTECT(NEQ = NEW_INTEGER(1));
+  PROTECT(LM = NEW_INTEGER(1));
 
-                              INTEGER(NEQ)[0] = *neq;
-                              INTEGER(LM) [0] = *lmas;
-  PROTECT(R_fcall = lang3(R_mas_func,NEQ,LM));   incr_N_Protect();
-  PROTECT(ans = eval(R_fcall, R_envir));         incr_N_Protect();
+  INTEGER(NEQ)[0] = *neq;
+  INTEGER(LM) [0] = *lmas;
+  PROTECT(R_fcall = lang3(R_mas_func,NEQ,LM));
+  PROTECT(ans = eval(R_fcall, R_envir));
 
   for (i = 0; i <*lmas * *neq; i++)   am[i] = REAL(ans)[i];
 
-  my_unprotect(4);
+  UNPROTECT(4);
 }
 
 /* deriv output function - for ordinary output variables                      */
 
-static void C_deriv_out_rad (int *nOut, double *t, double *y, 
+static void C_deriv_out_rad (int *nOut, double *t, double *y,
                        double *ydot, double *yout)
 {
   int i;
   SEXP R_fcall, Time, ans;
-  
-  for (i = 0; i < n_eq; i++)  
+
+  for (i = 0; i < n_eq; i++)
       REAL(Y)[i] = y[i];
-     
-  PROTECT(Time = ScalarReal(*t));                   incr_N_Protect();
-  PROTECT(R_fcall = lang3(R_deriv_func,Time, Y));   incr_N_Protect();
-  PROTECT(ans = eval(R_fcall, R_envir));            incr_N_Protect();
+
+  PROTECT(Time = ScalarReal(*t));
+  PROTECT(R_fcall = lang3(R_deriv_func,Time, Y));
+  PROTECT(ans = eval(R_fcall, R_envir));
 
   for (i = 0; i < *nOut; i++) yout[i] = REAL(ans)[i + n_eq];
 
-  my_unprotect(3);                                  
-}      
+  UNPROTECT(3);
+}
 
 /* save output in R-variables                                                 */
 
 static void saveOut (double t, double *y) {
   int j;
-  
+
     REAL(YOUT)[(it)*(ntot+1)] = t;
 	  for (j = 0; j < n_eq; j++)
 	    REAL(YOUT)[(it)*(ntot + 1) + j + 1] = y[j];
@@ -142,9 +142,9 @@ static void saveOut (double t, double *y) {
         deriv_func (&n_eq, &t, y, xdytmp, out, ipar) ;
       else
         C_deriv_out_rad(&nout, &t, y, xdytmp, out);
-      for (j = 0; j < nout; j++) 
+      for (j = 0; j < nout; j++)
         REAL(YOUT)[(it)*(ntot + 1) + j + n_eq + 1] = out[j];
-    }                
+    }
 }
 
 /* save lagged variables                                                      */
@@ -172,13 +172,13 @@ static void C_root_radau (int *neq, double *t, double *y, int *ng, double *gout)
 
   for (i = 0; i < *neq; i++)  REAL(Y)[i] = y[i];
 
-  PROTECT(Time = ScalarReal(*t));                 incr_N_Protect();
-  PROTECT(R_fcall = lang3(R_root_func,Time,Y));   incr_N_Protect();
-  PROTECT(ans = eval(R_fcall, R_envir));          incr_N_Protect();
+  PROTECT(Time = ScalarReal(*t));
+  PROTECT(R_fcall = lang3(R_root_func,Time,Y));
+  PROTECT(ans = eval(R_fcall, R_envir));
 
   for (i = 0; i < *ng; i++)   gout[i] = REAL(ans)[i];
 
-  my_unprotect(3);
+  UNPROTECT(3);
 }
 /* function for brent's root finding algorithm                                */
 
@@ -250,7 +250,7 @@ static void C_soloutrad(int * nr, double * told, double * t, double * y,
           tprevroot = tmin;
           for (j = 0; j < n_eq; j++) y[j] = ytmp[j];
         }
-       } 
+       }
      } else jroot[i] = 0;
     for (i = 0; i < nroot; i++) oldroot[i] = root[i];
   }
@@ -261,7 +261,7 @@ static void C_soloutrad(int * nr, double * told, double * t, double * y,
     it++;
     if ( it >= maxt) break;
   }
-   if ((*irtrn == -1) && rootevent) {      
+   if ((*irtrn == -1) && rootevent) {
      *t = tmin;
      tin = *t;
      tEvent = tin;
@@ -280,7 +280,7 @@ static void C_soloutrad(int * nr, double * told, double * t, double * y,
        nr_root++;
        updateevent(&tin, y, &istate);
        endsim = 0;
-     } else {  
+     } else {
        endsim = 1;
      }
    }
@@ -296,13 +296,13 @@ static void C_jac_func_rad(int *neq, double *t, double *y, int *ml,
 
   for (i = 0; i < *neq; i++) REAL(Y)[i] = y[i];
 
-  PROTECT(Time = ScalarReal(*t));                 incr_N_Protect();
-  PROTECT(R_fcall = lang3(R_jac_func,Time,Y));    incr_N_Protect();
-  PROTECT(ans = eval(R_fcall, R_envir));          incr_N_Protect();
+  PROTECT(Time = ScalarReal(*t));
+  PROTECT(R_fcall = lang3(R_jac_func,Time,Y));
+  PROTECT(ans = eval(R_fcall, R_envir));
 
   for (i = 0; i < *neq * *nrowpd; i++)  pd[i] = REAL(ans)[i];
 
-  my_unprotect(2);
+  UNPROTECT(3);
 }
 
 /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -313,7 +313,7 @@ typedef void C_solout_type (int *, double *, double *, double *,
 typedef void C_mas_type (int *, double *, int *, double *, int *);
 
 // to be changed...
-typedef void C_jac_func_type_rad(int *, double *, double *, int *, 
+typedef void C_jac_func_type_rad(int *, double *, double *, int *,
                      int *, double *, int*, double *, int *);
 
 
@@ -321,10 +321,10 @@ typedef void C_jac_func_type_rad(int *, double *, double *, int *,
                   MAIN C-FUNCTION, CALLED FROM R-code                       */
 
 SEXP call_radau(SEXP y, SEXP times, SEXP derivfunc, SEXP masfunc, SEXP jacfunc,
-    SEXP parms, SEXP rtol, SEXP atol, 
+    SEXP parms, SEXP rtol, SEXP atol,
     SEXP Nrjac, SEXP Nrmas,
 		SEXP rho, SEXP initfunc, SEXP rWork, SEXP iWork,
-    SEXP nOut, SEXP lRw, SEXP lIw, 
+    SEXP nOut, SEXP lRw, SEXP lIw,
     SEXP Rpar, SEXP Ipar, SEXP Hini, SEXP flist, SEXP elag,
     SEXP rootfunc, SEXP nRoot, SEXP eventfunc, SEXP elist )
 
@@ -338,6 +338,7 @@ SEXP call_radau(SEXP y, SEXP times, SEXP derivfunc, SEXP masfunc, SEXP jacfunc,
   int  isForcing;
   double *xytmp, tout, *Atol, *Rtol, hini=0;
   int itol, iout, idid;
+  int nprot = 0;
 
   SEXP TROOT, NROOT, VROOT, IROOT;
 
@@ -352,24 +353,23 @@ SEXP call_radau(SEXP y, SEXP times, SEXP derivfunc, SEXP masfunc, SEXP jacfunc,
 /*                      #### initialisation ####                              */
 
   lock_solver(); /* prevent nested call of solvers that have global variables */
-  long int old_N_Protect = save_N_Protected();
 
-  n_eq = LENGTH(y);             /* number of equations */ 
+  n_eq = LENGTH(y);             /* number of equations */
   nt   = LENGTH(times);         /* number of output times */
-  maxt = nt; 
+  maxt = nt;
   nroot  = INTEGER(nRoot)[0];   /* number of roots  */
   isroot = 0; nr_root = 0;
   if (nroot > 0) isroot = 1;
-  
+
   tt = (double *) R_alloc(nt, sizeof(double));
   for (j = 0; j < nt; j++) tt[j] = REAL(times)[j];
-  
+
   ijac  = INTEGER(Nrjac)[0];
-  mljac = INTEGER(Nrjac)[1]; 
-  mujac = INTEGER(Nrjac)[2]; 
+  mljac = INTEGER(Nrjac)[1];
+  mujac = INTEGER(Nrjac)[2];
   imas  = INTEGER(Nrmas)[0];
-  mlmas = INTEGER(Nrmas)[1]; 
-  mumas = INTEGER(Nrmas)[2]; 
+  mlmas = INTEGER(Nrmas)[1];
+  mumas = INTEGER(Nrmas)[2];
   /* is function a dll ?*/
   isDll = inherits(derivfunc, "NativeSymbol");
 
@@ -381,7 +381,7 @@ SEXP call_radau(SEXP y, SEXP times, SEXP derivfunc, SEXP masfunc, SEXP jacfunc,
   for (j = 0; j < n_eq; j++) xytmp[j] = REAL(y)[j];
 
   ytmp = (double *) R_alloc(n_eq, sizeof(double));
- 
+
   latol = LENGTH(atol);
   Atol = (double *) R_alloc((int) latol, sizeof(double));
   for (j = 0; j < latol; j++) Atol[j] = REAL(atol)[j];
@@ -393,9 +393,9 @@ SEXP call_radau(SEXP y, SEXP times, SEXP derivfunc, SEXP masfunc, SEXP jacfunc,
   /* tolerance specifications */
   if (latol == 1 ) itol = 0;
   else             itol = 1;
-  
+
   hini = REAL(Hini)[0];
-  
+
   /* work vectors */
   liw = INTEGER (lIw)[0];
   iwork = (int *) R_alloc(liw, sizeof(int));
@@ -408,25 +408,38 @@ SEXP call_radau(SEXP y, SEXP times, SEXP derivfunc, SEXP masfunc, SEXP jacfunc,
   for (j=length(rWork); j<lrw; j++) rwork[j] = 0.;
 
   /* initialise global R-variables...  */
-  initglobals (nt, ntot);
+  //initglobals (nt, ntot);
+  PROTECT(Y = allocVector(REALSXP, (n_eq))); nprot++;
+  PROTECT(YOUT = allocMatrix(REALSXP, ntot+1, nt)); nprot++;
+
   //timesteps = (double *) R_alloc(2, sizeof(double));
   for (j=0; j<2; j++) timesteps[j] = 0.;
-  
+
   /* Initialization of Parameters, Forcings (DLL), lags */
-  initParms (initfunc, parms);
+  //initParms(initfunc, parms);
+  if (initfunc != NA_STRING) {
+    if (inherits(initfunc, "NativeSymbol")) {
+      init_func_type *initializer;
+      PROTECT(de_gparms = parms); nprot++;
+      initializer = (init_func_type *) R_ExternalPtrAddrFn_(initfunc);
+      initializer(Initdeparms);
+    }
+  }
+  // end inline initParms
+
   isForcing = initForcings(flist);
   isEvent = initEvents(elist, eventfunc, nroot);
   islag = initLags(elag, 10, nroot);
 
   if (nout > 0 || islag) {
      xdytmp= (double *) R_alloc(n_eq, sizeof(double));
-     for (j = 0; j < n_eq; j++) xdytmp[j] = 0.; 
+     for (j = 0; j < n_eq; j++) xdytmp[j] = 0.;
   }
-  
+
  /* pointers to functions deriv_func, jac_func, passed to FORTRAN */
   if (isDll)  { /* DLL address passed to FORTRAN */
-      deriv_func = (C_deriv_func_type *) R_ExternalPtrAddrFn_(derivfunc);  
-	  
+      deriv_func = (C_deriv_func_type *) R_ExternalPtrAddrFn_(derivfunc);
+
  	   /* overruling deriv_func if forcing */
       if (isForcing) {
         DLL_deriv_func = deriv_func;
@@ -434,7 +447,7 @@ SEXP call_radau(SEXP y, SEXP times, SEXP derivfunc, SEXP masfunc, SEXP jacfunc,
       }
   } else {
       /* interface function between FORTRAN and C/R passed to FORTRAN */
-      deriv_func = (C_deriv_func_type *) C_deriv_func_rad; 
+      deriv_func = (C_deriv_func_type *) C_deriv_func_rad;
       /* needed to communicate with R */
       R_deriv_func = derivfunc;
   }
@@ -451,23 +464,23 @@ SEXP call_radau(SEXP y, SEXP times, SEXP derivfunc, SEXP masfunc, SEXP jacfunc,
   if (!isNull(masfunc))   {
 	   R_mas_func = masfunc;
 	   mas_func= C_mas_func_rad;
-     if (isDll)       
+     if (isDll)
        R_envir = rho;
 
   }
 
  	solout = C_soloutrad;
- 
+
   iout = 2;                           /* solout called after each step OR 1???*/
   idid = 0;
 
-/*                   ####      integration     ####                           */    
+/*                   ####      integration     ####                           */
   it   = 0;
   tin  = REAL(times)[0];
   tout = REAL(times)[nt-1];
-  saveOut (tin, xytmp);               /* save initial condition */ 
+  saveOut (tin, xytmp);               /* save initial condition */
   it++;
-  
+
   if (nroot > 0)  {      /* also must find a root */
     jroot = (int *) R_alloc(nroot, sizeof(int));
     for (j = 0; j < nroot; j++) jroot[j] = 0;
@@ -494,55 +507,60 @@ SEXP call_radau(SEXP y, SEXP times, SEXP derivfunc, SEXP masfunc, SEXP jacfunc,
   do {
     if (islag == 1) C_saveLag(1, &tin, xytmp, out, ipar, out, ipar);
 
-    F77_CALL(radau5) ( &n_eq, deriv_func, &tin, xytmp, &tout, &hini, 
-		     Rtol, Atol, &itol, jac_func, &ijac, &mljac, &mujac, 
+    F77_CALL(radau5) ( &n_eq, deriv_func, &tin, xytmp, &tout, &hini,
+		     Rtol, Atol, &itol, jac_func, &ijac, &mljac, &mujac,
          mas_func, &imas, &mlmas, &mumas, solout, &iout,
 		     rwork, &lrw, iwork, &liw, out, ipar, &idid);
 	} while (tin < tout && idid >= 0 && endsim == 0);
 
   if (idid == -1)
      warning("input is not consistent");
-  else if (idid == -2)   
+  else if (idid == -2)
      warning("larger maxsteps needed");
-  else if (idid == -3)   
+  else if (idid == -3)
      warning("step size becomes too small");
-  else if (idid == -4)   
+  else if (idid == -4)
      warning("problem is probably stiff - interrupted");
 
-/*                   ####  an error occurred   ####                           */    
+/*                   ####  an error occurred   ####                           */
   if(it <= nt-1) saveOut (tin, xytmp);              /* save final condition */
   if (idid < 0) {
     it = it-1;
+    PROTECT(YOUT2 = allocMatrix(REALSXP,ntot+1,(it+2))); nprot++;
     returnearly (1, it, ntot);
   } else if (idid == 2) {
     it = it-1;
+	PROTECT(YOUT2 = allocMatrix(REALSXP,ntot+1,(it+2))); nprot++;   
     returnearly (0, it, ntot);
     idid = -2;
   }
-/*                   ####   returning output   ####                           */    
+/*                   ####   returning output   ####                           */
   rwork[0] = hini;
-  rwork[1] = tin ; 
-  terminate(idid,iwork,7,13,rwork,5,0);       
+  rwork[1] = tin ;
+
+  PROTECT(ISTATE = allocVector(INTSXP, 7)); nprot++;
+  PROTECT(RWORK = allocVector(REALSXP, 5)); nprot++;
+  terminate(idid,iwork,7,13,rwork,5,0);
 
   if (iroot >= 0 || nr_root > 0)  {
-    PROTECT(IROOT = allocVector(INTSXP, nroot));incr_N_Protect();
+    PROTECT(IROOT = allocVector(INTSXP, nroot)); nprot++;
     for (j = 0; j < nroot; j++) INTEGER(IROOT)[j] = jroot[j];
-    PROTECT(NROOT = allocVector(INTSXP, 1));incr_N_Protect();
+    PROTECT(NROOT = allocVector(INTSXP, 1)); nprot++;
     INTEGER(NROOT)[0] = nr_root;
 
     if (nr_root == 0) {
-      PROTECT(TROOT = allocVector(REALSXP, 1)); incr_N_Protect();
+      PROTECT(TROOT = allocVector(REALSXP, 1)); nprot++;
       REAL(TROOT)[0] = tin;
     } else {
       if (nr_root > Rootsave) nr_root = Rootsave;
 
-      PROTECT(TROOT = allocVector(REALSXP, nr_root)); incr_N_Protect();
+      PROTECT(TROOT = allocVector(REALSXP, nr_root)); nprot++;
       for (j = 0; j < nr_root; j++) REAL(TROOT)[j] = troot[j];
 
-      PROTECT(VROOT = allocVector(REALSXP, nr_root*n_eq)); incr_N_Protect();
+      PROTECT(VROOT = allocVector(REALSXP, nr_root*n_eq)); nprot++;
       for (j = 0; j < nr_root*n_eq; j++) REAL(VROOT)[j] = valroot[j];
 
-      PROTECT(IROOT = allocVector(INTSXP, nr_root)); incr_N_Protect();
+      PROTECT(IROOT = allocVector(INTSXP, nr_root)); nprot++;
       for (j = 0; j < nr_root; j++) INTEGER(IROOT)[j] = nrroot[j];
 
       if (idid == 1) {
@@ -564,13 +582,26 @@ SEXP call_radau(SEXP y, SEXP times, SEXP derivfunc, SEXP masfunc, SEXP jacfunc,
       setAttrib(YOUT2, install("nroot"), NROOT);
     }
   }
-/*                   ####     termination      ####                           */    
+
+/*                   ####     termination      ####                           */
   unlock_solver();
-  restore_N_Protected(old_N_Protect);                           
-  //unprotect_all();
+  UNPROTECT(nprot);
+  						
+// thpe: after reworking PROTECT/UNPROTECT, I checked how YOUT, YOUT2 is handled
+// and see that the following is not consistent, because YOUT is only set when idid==1
+// Is this a (still) hidden bug?
+//
+
+// original version
+//  if (idid > 0)
+//    return(YOUT);
+//  else
+//    return(YOUT2);
+
+// thpe: test version (currently disabled)
   if (idid > 0)
     return(YOUT);
   else
-    return(YOUT2);
+    return(YOUT2);	
 }
- 
+
